package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.StandardMaterialApprovalForm;
import digital.laboratory.platform.reagent.service.StandardMaterialApprovalFormService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (标准物质停用/报废销毁/恢复/降级使用审批表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质停用/报废销毁/恢复/降级使用审批表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/standard_material_approval_form" )
@Api(value = "standard_material_approval_form", tags = "(标准物质停用/报废销毁/恢复/降级使用审批表)管理")
public class StandardMaterialApprovalFormController {

    private final  StandardMaterialApprovalFormService standardMaterialApprovalFormService;

	/**
	 * 通过id查询(标准物质停用/报废销毁/恢复/降级使用审批表)
	 * @param standardMaterialApprovalFormId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{standardMaterialApprovalFormId}" )
	@PreAuthorize("@pms.hasPermission('reagent_standard_material_approval_form_get')" )
	public R<StandardMaterialApprovalForm> getById(@PathVariable("standardMaterialApprovalFormId" ) String standardMaterialApprovalFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StandardMaterialApprovalForm standardMaterialApprovalForm = standardMaterialApprovalFormService.getById(standardMaterialApprovalFormId);
		return R.ok(standardMaterialApprovalForm);
		//return R.ok(standardMaterialApprovalFormService.getById(standardMaterialApprovalFormId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param standardMaterialApprovalForm (标准物质停用/报废销毁/恢复/降级使用审批表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_approval_form_get')" )
    public R<IPage<StandardMaterialApprovalForm>> getStandardMaterialApprovalFormPage(Page<StandardMaterialApprovalForm> page, StandardMaterialApprovalForm standardMaterialApprovalForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<StandardMaterialApprovalForm> standardMaterialApprovalFormSList = standardMaterialApprovalFormService.page(page, Wrappers.<StandardMaterialApprovalForm>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(standardMaterialApprovalFormSList);
//		return R.ok(standardMaterialApprovalFormService.page(page, Wrappers.query(standardMaterialApprovalForm)));
    }


    /**
     * 新增(标准物质停用/报废销毁/恢复/降级使用审批表)
     * @param standardMaterialApprovalForm (标准物质停用/报废销毁/恢复/降级使用审批表)
     * @return R
     */
    @ApiOperation(value = "新增(标准物质停用/报废销毁/恢复/降级使用审批表)", notes = "新增(标准物质停用/报废销毁/恢复/降级使用审批表)")
    @SysLog("新增(标准物质停用/报废销毁/恢复/降级使用审批表)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_approval_form_add')" )
    public R<StandardMaterialApprovalForm> postAddObject(@RequestBody StandardMaterialApprovalForm standardMaterialApprovalForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        standardMaterialApprovalForm.setStandardMaterialApprovalFormId(IdWorker.get32UUID().toUpperCase());
		if (standardMaterialApprovalFormService.save(standardMaterialApprovalForm)) {
			return R.ok(standardMaterialApprovalForm, "对象创建成功");
		}
		else {
			return R.failed(standardMaterialApprovalForm, "对象创建失败");
		}
    }

    /**
     * 修改(标准物质停用/报废销毁/恢复/降级使用审批表)
     * @param standardMaterialApprovalForm (标准物质停用/报废销毁/恢复/降级使用审批表)
     * @return R
     */
    @ApiOperation(value = "修改(标准物质停用/报废销毁/恢复/降级使用审批表)", notes = "修改(标准物质停用/报废销毁/恢复/降级使用审批表)")
    @SysLog("修改(标准物质停用/报废销毁/恢复/降级使用审批表)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_approval_form_edit')" )
    public R<StandardMaterialApprovalForm> putUpdateById(@RequestBody StandardMaterialApprovalForm standardMaterialApprovalForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (standardMaterialApprovalFormService.updateById(standardMaterialApprovalForm)) {
			return R.ok(standardMaterialApprovalForm, "保存对象成功");
		}
		else {
			return R.failed(standardMaterialApprovalForm, "保存对象失败");
		}
    }

    /**
     * 通过id删除(标准物质停用/报废销毁/恢复/降级使用审批表)
     * @param standardMaterialApprovalFormId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(标准物质停用/报废销毁/恢复/降级使用审批表)", notes = "通过id删除(标准物质停用/报废销毁/恢复/降级使用审批表)")
    @SysLog("通过id删除(标准物质停用/报废销毁/恢复/降级使用审批表)" )
    @DeleteMapping("/{standardMaterialApprovalFormId}" )
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_approval_form_del')" )
    public R<StandardMaterialApprovalForm> deleteById(@PathVariable String standardMaterialApprovalFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StandardMaterialApprovalForm oldStandardMaterialApprovalForm = standardMaterialApprovalFormService.getById(standardMaterialApprovalFormId);

		if (standardMaterialApprovalFormService.removeById(standardMaterialApprovalFormId)) {
			return R.ok(oldStandardMaterialApprovalForm, "对象删除成功");
		}
		else {
			return R.failed(oldStandardMaterialApprovalForm, "对象删除失败");
		}

    }

}
